@@ -1,7 +1,9 @@
+use serde::{Deserialize, Serialize};
+
 use crate::rendering::bidimensional::gl_representations::GlColor;
 
 /// A struct that represents colors for rendering.
-#[derive(Clone)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct Color {
     /// Red value of the color
     r: u8,
