@@ -2,9 +2,10 @@
 
 pub use shapes::{square::Square, triangle::Triangle};
 
+pub mod animations;
 pub mod color;
 pub mod material;
 pub mod maths;
 pub mod shapes;
-pub mod sprite;
+pub mod tiles;
 pub mod ui;
